<?php

namespace Drupal\foldershare\Form\AdminSettingsTraits;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StreamWrapper\PrivateStream;

use Drupal\foldershare\ManageFilenameExtensions;
use Drupal\foldershare\ManageHooks;
use Drupal\foldershare\Settings;
use Drupal\foldershare\Utilities\LinkUtilities;
use Drupal\foldershare\Utilities\LimitUtilities;
use Drupal\foldershare\Entity\FolderShare;

/**
 * Manages the "Files" tab for the module's settings form.
 *
 * <B>Warning:</B> This is an internal trait that is strictly used by
 * the AdminSettings form class. It is a mechanism to group functionality
 * to improve code management.
 *
 * @ingroup foldershare
 */
trait AdminSettingsFilesTab {

  /*---------------------------------------------------------------------
   *
   * Build.
   *
   *---------------------------------------------------------------------*/
  /**
   * Builds the files tab.
   *
   * @param array $form
   *   An associative array containing the structure of the form. The form
   *   is modified to include additional render elements for the tab.
   * @param \Drupal\Core\Form\FormStateInterface $formState
   *   The current state of the form.
   * @param string $tabGroup
   *   The name of the tab group.
   */
  private function buildFilesTab(
    array &$form,
    FormStateInterface $formState,
    string $tabGroup) {

    //
    // Find installed modules.
    // -----------------------
    // Tab sections vary their presentation based upon what modules are
    // installed at the site.
    $helpInstalled = $this->moduleHandler->moduleExists('help');

    //
    // Set up tab names.
    // -----------------
    // Create the machine name for the tab and the tab's title.
    $tabMachineName = 'files';
    $tabName        = 'foldershare_' . $tabMachineName . '_tab';
    $tabTitle       = $this->t('Files');
    $tabPaneTitle   = $this->t('Manage files');

    //
    // Set up class names.
    // -------------------
    // Use a set of standard class names for tab sections.
    $tabSubtitleClass        = 'foldershare-settings-subtitle';
    $sectionWrapperClass     = 'foldershare-settings-section-wrapper';
    $sectionClass            = 'foldershare-settings-section';
    $sectionDescriptionClass = 'foldershare-settings-section-description';
    $itemDefaultClass        = 'foldershare-settings-item-default';
    $itemNoteClass           = 'foldershare-settings-item-note';
    $warningClass            = 'foldershare-warning';

    $restoreExtensionsButton = 'foldershare_extensions_restore';

    //
    // Create the tab
    // --------------
    // Start the tab with a title, subtitle, and description.
    $form[$tabName] = [
      '#type'                    => 'details',
      '#open'                    => FALSE,
      '#group'                   => $tabGroup,
      '#title'                   => $tabTitle,

      '#description'             => [
        'subtitle'               => [
          '#type'                => 'html_tag',
          '#tag'                 => 'h2',
          '#value'               => $tabPaneTitle,
          '#attributes'          => [
            'class'              => [$tabSubtitleClass],
          ],
        ],

        'description'            => [
          '#type'                => 'html_tag',
          '#tag'                 => 'p',
          '#value'               => $this->t(
            'Control where files are saved, the file name extensions supported, and the maximum file upload size.'),
          '#attributes'          => [
            'class'              => [
              'foldershare-settings-description',
            ],
          ],
        ],
      ],
      '#attributes'              => [
        'class'                  => [
          'foldershare-settings-tab',
          'foldershare-files-tab',
        ],
      ],
    ];

    //
    // File system.
    // ------------
    // If the site currently has stored files, then the storage scheme
    // and subdirectory choice cannot be changed.
    //
    // If the site doesn't have stored files, but the private file system
    // has not been configured, then the storage scheme cannot be changed.
    $inUseWarning          = '';
    $noPrivateWarning      = '';
    $storageSchemeDisabled = FALSE;

    if (FolderShare::hasFiles() === TRUE) {
      // Create a link to the Drupal core system page to delete all content
      // for the entity type.
      $inUseWarning = [
        '#type'                  => 'html_tag',
        '#tag'                   => 'p',
        '#value'                 => $this->t(
          'This setting is disabled because there are still files managed by this module. You must @deleteAllLink files first.',
          [
            '@deleteAllLink'     => LinkUtilities::createRouteLink(
              'system.prepare_modules_entity_uninstall',
              '',
              $this->t('Delete all'),
              [
                'entity_type_id' => FolderShare::ENTITY_TYPE_ID,
              ]),
          ]),
        '#attributes'            => [
          'class'                => [
            $warningClass,
            $itemNoteClass,
          ],
        ],
      ];
      $storageSchemeDisabled = TRUE;
    }

    if (empty(PrivateStream::basePath()) === TRUE) {
      if ($helpInstalled === TRUE) {
        $noPrivateWarning = [
          '#type'                  => 'html_tag',
          '#tag'                   => 'p',
          '#value'                 => $this->t(
            'This setting is limited to the <em>Public</em> directory because a <em>Private</em> directory has not been set up yet (see @fileHelpLink and @fileDocLink, @fileSettingsLink settings, and the "file_private_path" setting in the site\'s "settings.php" file).',
            [
              '@fileSettingsLink'  => LinkUtilities::createRouteLink(
                'system.file_system_settings',
                '',
                $this->t('File system')),
              '@fileHelpLink'      => LinkUtilities::createHelpLink(
                'file',
                $this->t('File help')),
              '@fileDocLink'       => LinkUtilities::createDocLink(
                'file',
                $this->t('File documentation')),
            ]),
          '#attributes'            => [
            'class'                => [
              $warningClass,
              $itemNoteClass,
            ],
          ],
        ];
      }
      else {
        $noPrivateWarning = [
          '#type'                  => 'html_tag',
          '#tag'                   => 'p',
          '#value'                 => $this->t(
            'This setting is limited to the <em>Public</em> directory because a <em>Private</em> directory has not been set up yet (see @fileDocLink, @fileSettingsLink settings, and the "file_private_path" setting in the site\'s "settings.php" file).',
            [
              '@fileSettingsLink'  => LinkUtilities::createRouteLink(
                'system.file_system_settings',
                '',
                $this->t('File system')),
              '@fileDocLink'       => LinkUtilities::createDocLink(
                'file',
                $this->t('File documentation')),
            ]),
          '#attributes'            => [
            'class'                => [
              $warningClass,
              $itemNoteClass,
            ],
          ],
        ];
      }
      $storageSchemeDisabled = TRUE;
    }

    $options = [
      'public'  => $this->t('Public directory'),
      'private' => $this->t('Private directory'),
    ];

    $form[$tabName]['directories'] = [
      '#type'                    => 'details',
      '#title'                   => $this->t('Directories'),
      '#open'                    => FALSE,
      '#attributes'              => [
        'class'                  => [$sectionWrapperClass],
      ],

      'section'                  => [
        '#type'                  => 'container',
        '#attributes'            => [
          'class'                => [$sectionClass],
        ],

        'section-description'    => [
          '#type'                => 'html_tag',
          '#tag'                 => 'p',
          '#value'               => $this->t(
            "Store files locally in the site's <em>Public</em> or <em>Private</em> directories. <em>Private</em> directories provide better security."),
          '#attributes'          => [
            'class'              => [$sectionDescriptionClass],
          ],
        ],

        'directory-choice'       => [
          '#type'                => 'select',
          '#options'             => $options,
          '#default_value'       => Settings::getFileScheme(),
          '#disabled'            => $storageSchemeDisabled,
          '#name'                => 'directory-choice',
        ],

        // The default setting.
        'directory-choice-description' => [
          '#type'                => 'html_tag',
          '#tag'                 => 'p',
          '#value'               => $this->t(
            'Default: @default',
            [
              '@default'         => $options[Settings::getFileSchemeDefault()],
            ]),
          '#attributes'          => [
            'class'              => [
              'description',
              $itemDefaultClass,
              $itemNoteClass,
            ],
          ],
        ],
      ],
    ];

    // Add warning if there are things disabled.
    if (empty($noPrivateWarning) === FALSE) {
      $form[$tabName]['directories']['section']['warning'] = $noPrivateWarning;
    }
    elseif (empty($inUseWarning) === FALSE) {
      $form[$tabName]['directories']['section']['warning'] = $inUseWarning;
    }

    //
    // File name extensions
    // --------------------
    // Select whether file uploads should be restricted based on their
    // file name extensions, and what extensions are allowed.
    $fileRestrictExtensions = Settings::getFileRestrictFilenameExtensions();
    $fileRestrictExtensionsDefault = Settings::getFileRestrictFilenameExtensionsDefault();

    // Create a default value message.
    if ($fileRestrictExtensionsDefault === TRUE) {
      $defaultMessage = $this->t('Default: Restrict files to a list of allowed extensions');
    }
    else {
      $defaultMessage = $this->t('Default: Allow all extensions');
    }

    // Create a hook override message.
    $hookModules = ManageHooks::getHookAllowedFilenameExtensionsAlter();
    if (empty($hookModules) === TRUE) {
      $hookModulesMessage = '';
    }
    elseif (count($hookModules) === 1) {
      $hookModulesMessage = $this->t(
        "The %name module has installed an override that can limit extensions for specific folders or users.",
        [
          '%name' => $this->moduleHandler->getName($hookModules[0]),
        ]);
    }
    else {
      $names = [];
      foreach ($hookModules as $moduleName) {
        $names[] = $this->moduleHandler->getName($moduleName);
      }
      $hookModulesMessage = $this->t(
        "@num modules have installed overrides that can limit extensions for specific folders or users: %list.",
        [
          '@num'  => count($names),
          '%list' => implode(', ', $names),
        ]);
    }

    $form[$tabName]['fileextensions'] = [
      '#type'                    => 'details',
      '#title'                   => $this->t('File name extensions'),
      '#open'                    => FALSE,
      '#attributes'              => [
        'class'                  => [$sectionWrapperClass],
      ],
      'section'                  => [
        '#type'                  => 'container',
        '#attributes'            => [
          'class'                => [$sectionClass],
        ],

        'section-description'    => [
          '#type'                => 'html_tag',
          '#tag'                 => 'p',
          '#value'               => $this->t(
            'Allow all file name extensions or restrict them.'),
          '#attributes'          => [
            'class'              => [$sectionDescriptionClass],
          ],
        ],

        // A checkbox to enable/disable allowing all extensions.
        'all-file-extensions'    => [
          '#type'                => 'checkbox',
          '#title'               => $this->t('Allow all extensions'),
          '#default_value'       => ($fileRestrictExtensions === FALSE),
          '#return_value'        => 'enabled',
          '#name'                => 'all-file-extensions',
        ],

        // The default setting.
        'all-file-extensions-default' => [
          '#type'                => 'html_tag',
          '#tag'                 => 'p',
          '#value'               => $defaultMessage,
          '#attributes'          => [
            'class'              => [
              'description',
              $itemDefaultClass,
              $itemNoteClass,
            ],
          ],
        ],

        // Specific extensions.
        'set-extensions'         => [
          '#type'                => 'container',
          '#states'              => [
            'visible'            => [
              'input[name="all-file-extensions"]' => [
                'checked'        => FALSE,
              ],
            ],
          ],

          // A list of allowed extensions.
          'file-extensions'      => [
            '#type'              => 'textarea',
            '#title'             => $this->t('Allowed extensions:'),
            '#default_value'     => Settings::getAllowedFilenameExtensions(),
            '#required'          => FALSE,
            '#rows'              => 10,
            '#name'              => 'file-extensions',
            '#description'       => $this->t(
              'Separate extensions with spaces. Do not include dots. Changing this list does not affect files that are already on the site.'),
            '#attributes'        => [
              // No browsers support this. Disable autocomplete.
              'autocomplete' => 'off',
              // Most browsers support this. Disables automatic capitalization.
              'autocapitalize' => 'none',
              // Most browsers support this. Disables spell check.
              'spellcheck'   => 'false',
              // Safari only supports this. Disables spell check.
              'autocorrect'  => 'off',
              // Firefox for Android only. Names the 'Enter' key on a
              // virtual keybd.
              'mozactionhint' => $this->t('Change'),
            ],
          ],

          // A button to restore the default configuration.
          $restoreExtensionsButton => [
            '#type'              => 'submit',
            '#value'             => $this->t('Restore'),
            '#name'              => $restoreExtensionsButton,
            '#attributes'        => [
              'class'            => [
                $restoreExtensionsButton,
              ],
            ],
          ],

          // A description for the restore button.
          'extensions-restore-description' => [
            '#type'              => 'html_tag',
            '#tag'               => 'p',
            '#value'             => $this->t('Restore extensions to default values.'),
            '#attributes'        => [
              'class'            => [
                $restoreExtensionsButton . '_description',
              ],
            ],
          ],
        ],

        // A list of modules that override the extension list.
        'file-extensions-hooks' => [
          '#type'              => 'html_tag',
          '#tag'               => 'p',
          '#value'             => $hookModulesMessage,
          '#attributes'          => [
            'class'              => [
              $itemNoteClass,
            ],
          ],
        ],
      ],
    ];

    if (empty($hookModulesMessage) === TRUE) {
      unset($form[$tabName]['fileextensions']['section']['file-extensions-hooks']);
    }

    // Get the current extensions, if any, on the form.
    $value = $formState->getValue('file-extensions');
    if ($value !== NULL) {
      $form[$tabName]['fileextensions']['section']['file-extensions']['#value'] = $value;
    }

    //
    // File upload size
    // ----------------
    // Select whether file uploads should be restricted based on their size,
    // and what maximum size is allowed.
    $fileUploadSizeLimit = Settings::getFileUploadSizeLimit();
    $fileUploadSizeLimitDefault = Settings::getFileUploadSizeLimitDefault();
    $phpFileUploadSizeLimit = LimitUtilities::getPhpFileUploadSizeLimit();
    $formattedPhpFileUploadSizeLimit = number_format($phpFileUploadSizeLimit);

    // Create a default value message.
    if ($fileUploadSizeLimitDefault === 0) {
      $defaultMessage = $this->t(
        "Default: Allow any size up to the PHP maximum of @phplimit bytes.",
        [
          '@phplimit' => $formattedPhpFileUploadSizeLimit,
        ]);
    }
    else {
      $defaultMessage = $this->t(
        'Default: @bytes bytes',
        ['@bytes' => $fileUploadSizeLimitDefault]);
    }

    // Create a hook override message.
    $hookModules = ManageHooks::getHookFileUploadSizeLimitAlter();
    if (empty($hookModules) === TRUE) {
      $hookModulesMessage = '';
    }
    elseif (count($hookModules) === 1) {
      $hookModulesMessage = $this->t(
        "The %name module has installed an override that can further limit upload sizes for specific folders or users.",
        [
          '%name' => $this->moduleHandler->getName($hookModules[0]),
        ]);
    }
    else {
      $names = [];
      foreach ($hookModules as $moduleName) {
        $names[] = $this->moduleHandler->getName($moduleName);
      }
      $hookModulesMessage = $this->t(
        "@num modules have installed overrides that can further limit upload sizes for specific folders or users: %list.",
        [
          '@num'  => count($names),
          '%list' => implode(', ', $names),
        ]);
    }

    $form[$tabName]['fileuploadsize'] = [
      '#type'                    => 'details',
      '#title'                   => $this->t('File uploads'),
      '#open'                    => FALSE,
      '#attributes'              => [
        'class'                  => [$sectionWrapperClass],
      ],
      'section'                  => [
        '#type'                  => 'container',
        '#attributes'            => [
          'class'                => [$sectionClass],
        ],

        'section-description'    => [
          '#type'                => 'html_tag',
          '#tag'                 => 'p',
          '#value'               => $this->t(
            'Allow any file size or restrict uploads to a maximum size.'),
          '#attributes'          => [
            'class'              => [$sectionDescriptionClass],
          ],
        ],

        // A checkbox to enable/disable limiting file sizes.
        'any-file-upload-size'    => [
          '#type'                => 'checkbox',
          '#title'               => $this->t('Allow any file upload size'),
          '#default_value'       => ($fileUploadSizeLimit === 0),
          '#return_value'        => 'enabled',
          '#name'                => 'any-file-upload-size',
        ],

        // The default setting.
        'any-file-upload-size-default' => [
          '#type'                => 'html_tag',
          '#tag'                 => 'p',
          '#value'               => $defaultMessage,
          '#attributes'          => [
            'class'              => [
              'description',
              $itemDefaultClass,
              $itemNoteClass,
            ],
          ],
        ],

        // Specific limit.
        'set-file-upload-size'   => [
          '#type'                => 'container',
          '#states'              => [
            'visible'            => [
              'input[name="any-file-upload-size"]' => [
                'checked'        => FALSE,
              ],
            ],
          ],

          // The upload size limit.
          'file-upload-size'     => [
            '#type'              => 'number',
            '#title'             => $this->t('Maximum file size:'),
            '#default_value'     => $fileUploadSizeLimit,
            '#required'          => FALSE,
            '#min'               => 0,
            '#max'               => $phpFileUploadSizeLimit,
            '#step'              => 1,
            '#size'              => 24,
            '#name'              => 'file-upload-size',
            '#description'       => $this->t(
              "Enter a size up to the PHP maximum. A zero uses the maximum. Changing the size does not affect files that are already on the site."),
          ],
        ],

        // A note that PHP still limits the size.
        'file-upload-size-php' => [
          '#type'              => 'html_tag',
          '#tag'               => 'p',
          '#value'             => $this->t(
            "The PHP maximum is controlled by the <em>upload_max_filesize</em> directive of the server's <em>php.ini</em> configuration file. See the server administrator to update this file."),
          '#attributes'        => [
            'class'            => [
              $itemNoteClass,
            ],
          ],
        ],

        // A list of modules that override the extension list.
        'file-upload-size-hooks' => [
          '#type'                => 'html_tag',
          '#tag'                 => 'p',
          '#value'               => $hookModulesMessage,
          '#attributes'          => [
            'class'              => [
              $itemNoteClass,
            ],
          ],
        ],
      ],
    ];

    if (empty($hookModulesMessage) === TRUE) {
      unset($form[$tabName]['fileuploadsize']['section']['file-upload-size-hooks']);
    }
  }

  /*---------------------------------------------------------------------
   *
   * Validate.
   *
   *---------------------------------------------------------------------*/
  /**
   * Validates form values.
   *
   * @param array $form
   *   The form configuration.
   * @param \Drupal\Core\Form\FormStateInterface $formState
   *   The entered values for the form.
   */
  private function validateFilesTab(
    array &$form,
    FormStateInterface $formState) {

    //
    // Restore filename extensions.
    // ----------------------------
    // Restore the list of allowed file name extensions.
    $trigger = $formState->getTriggeringElement();
    $restoreExtensionsButton = 'foldershare_extensions_restore';

    if ($trigger['#name'] === $restoreExtensionsButton) {
      // Revert extensions configuration.
      $value = Settings::getAllowedFilenameExtensionsDefault();
      Settings::setAllowedFilenameExtensions($value);
      ManageFilenameExtensions::setAllowedFilenameExtensions($value);

      // Reset the form's extensions list.
      $formInput = $formState->getUserInput();
      $formInput['file-extensions'] = $value;
      $formState->setUserInput($formInput);

      // Rebuild the page.
      $formState->setRebuild(TRUE);
      $formState->cleanValues();
      $formState->setTriggeringElement(NULL);

      $this->messenger()->addStatus(
        $this->t('The file extensions list has been restored to its default values.'));
      return;
    }

    //
    // Directories.
    // ------------
    // It must be "public" or "private". If "private", the site's private
    // file system must have been configured.
    $hasPrivate = empty(PrivateStream::basePath()) === FALSE;

    $scheme = $formState->getValue('directory-choice');
    if ($scheme !== 'public' && $scheme !== 'private') {
      $formState->setErrorByName(
        'directory-choice',
        $this->t('The directory choice must be either <em>Public</em> or <em>Private</em>.'));
    }

    if ($scheme === 'private' && $hasPrivate === FALSE) {
      $formState->setErrorByName(
        'directory-choice',
        $this->t('The <em>Private</em> directory is not available because the site has not been configured to support it yet.'));
    }
  }

  /*---------------------------------------------------------------------
   *
   * Submit.
   *
   *---------------------------------------------------------------------*/
  /**
   * Stores submitted form values.
   *
   * @param array $form
   *   The form configuration.
   * @param \Drupal\Core\Form\FormStateInterface $formState
   *   The entered values for the form.
   */
  private function submitFilesTab(
    array &$form,
    FormStateInterface $formState) {

    //
    // File directory.
    // ---------------
    // If the site does not have any files, update the directory choice.
    if (FolderShare::hasFiles() === FALSE) {
      $value = $formState->getValue('directory-choice');
      Settings::setFileScheme($value);
    }

    //
    // File extensions.
    // ----------------
    // Set the flag allowing all extensions or not, and the extension list.
    // Note that the checkbox is TRUE to *allow*, while the setting has the
    // reverse sense and is TRUE to *restrict*.
    $value = $formState->getValue('all-file-extensions');
    $allowAll = ($value === 'enabled') ? TRUE : FALSE;
    Settings::setFileRestrictFilenameExtensions($allowAll === FALSE);

    // Specific extensions, set only if not allowing all extensions.
    if ($allowAll === FALSE) {
      $extensions = $formState->getValue('file-extensions');
      Settings::setAllowedFilenameExtensions($extensions);
    }

    // Forward extensions to the file field.
    if ($allowAll === TRUE) {
      // Everything allowed. No extensions list.
      ManageFilenameExtensions::setAllowedFilenameExtensions('');
    }
    else {
      ManageFilenameExtensions::setAllowedFilenameExtensions(
        Settings::getAllowedFilenameExtensions());
    }

    //
    // File upload size limit.
    // -----------------------
    // If any size is allowed, revert to a zero to mean use the PHP max.
    // Otherwise update the limit using the user's value. The setting
    // automatically restricts the value to something sane.
    $value = $formState->getValue('any-file-upload-size');
    $allowAll = ($value === 'enabled') ? TRUE : FALSE;
    if ($allowAll === TRUE) {
      $fileUploadSize = 0;
    }
    else {
      $fileUploadSize = intval($formState->getValue('file-upload-size'));
    }

    Settings::setFileUploadSizeLimit($fileUploadSize);
  }

}
